// RTTransClientDLL.h : DLL AvP[Vp̃Gg |Cg`܂B
//
/**
 *
 * @date	2012-08-03
 * @author	Imazu.s
 * @note	add PvwLRTGetChParam(),IdlLRTGetChParam()
 */
#ifndef RTTRANSCLIENTDLL_H
#define RTTRANSCLIENTDLL_H

#ifndef	DllClass
	#ifdef BUILD_DLL
		#define DllClass __declspec( dllexport )
	#else
		#if ( defined(WIN32)||defined(WIN64) )&&!defined(_USRSLL)
			#define DllClass __declspec( dllimport )
		#else
			#define DllClass
		#endif
	#endif // BUILD_DLL
#endif	//	DllClass

#if defined(WIN64)
 typedef __int64		pvw_long;
 typedef __int64		pvw_ulong;
#elif defined(__LP64__)
 typedef long			pvw_long;
 typedef unsigned long	pvw_ulong;
#else
 typedef int			pvw_long;
 typedef unsigned int	pvw_ulong;
#endif
 typedef unsigned short  pvw_uint;
 typedef short  pvw_int;

#if defined(WIN32) || defined(WIN64)
 typedef 	__int64 int64_t;  
#else
 typedef	int 	SOCKET;
 typedef	int		BOOL;
 typedef 	void* 	LPVOID;  
#if defined(__LP64__) && !defined(__MACH__)
 typedef	long	int64_t;
#else
 typedef	long long	int64_t;
#endif
 #define	FALSE	0
 #define	TRUE	1
 #define	INVALID_SOCKET	-1
 #define	SOCKET_ERROR	-1
 
#endif

typedef	short	idl_int;
typedef	int		idl_long;
typedef	int64_t	idl_long64;

//------------------------------------------------------------------------------
//   Ver. 3.0 2019-09
//------------------------------------------------------------------------------
#ifdef __cplusplus
extern "C"
{
#endif
 DllClass int RTCStart( void); 
 DllClass int RTCEnd( void); 
/** RTCopen
 *
 * @param [in] diag		Diagnostics name, If not specified, Set null or empty string("").
 * @param [in] server	Server name, If not specified, Set null or empty string("").
 * @param [in] channel_list
 *						Channel number list string.
 *             When specifying multiple numbers, connect the numbers with commas.
 *             When specifying a range, the format is "start number: end number".
 *             Multiple specification and range specification can be used together. In that case, connect with a comma.
 *
 * @param [in] control_port
 *                      Control TCP port number, Set '-1' when using the system setting value.
 *
 * @param [in] packet_size
 *                      If >0, specify data transfer UDP connection and its packet size.
 *                          In the case of MAX_INT, the packet size is the system maximum value.
 *                      If ==0, specify data transfer TCP connection.
 *
 * @param [in] thinning  Thinning rate, 1/N
 * @param [in] thin_type Thinning type, If ==0, it is a sample type. If ==1, it is a block type.  
 *
 * @return If >0, it is a descriptor for real-time data acquisition.
 *         If <0, an error has occurred.
 * *

 * @note 
 *   Either 'diag' or 'server' must be set.
 *   If both 'diag' and 'server' are set, 'server' takes precedence.
 */
 DllClass int RTCopen( char* diag, char* server, char* channel_list, 
					   int control_port, int packet_size, int thinning, int thin_type );
/** RTCchannel
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] channel_list See RTCopen().
 * @param [in] packet_size  See RTCopen().
 * @param [in] thinning     See RTCopen().
 * @param [in] thin_type    See RTCopen().  
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 */
 DllClass int RTCchannel( int desc, char* channel_list, int packet_size, int thinning, int thin_type );

/** RTCgetChannelParam
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] chno     Channel number
 * @param [out] prm     Channel parameter 
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 * @note
 *      An error occurs when the experimental state is "Idle".
 *
 */
 typedef struct {
	int samplingInterval;	// Sampling interval(nanosecond)
	int valueLen;			// Data bit length
	int	recLen;				// Data acquisition block byte length
 } RTC_CH_PARAM;

 DllClass int RTCgetChannelParam( int desc, int chno, RTC_CH_PARAM* prm);
 DllClass int RTCgetChannelParam2( int desc, int chno, 
								int* interval, int* valuelen, int* reclen);

/** RTCgetCamParam
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] chno     Channel number
 * @param [out] prm     Channel parameter 
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 * @note
 *      An error occurs when the experimental state is "Idle".
 *
 */
 typedef struct {
	int	framerate;
	int pixeldipth;
	int reclen;
	int framewidth;
	int frameheight;
 } RTC_CAM_PARAM;

 DllClass int RTCgetCamParam( int desc, int chno, RTC_CAM_PARAM* prm);
 DllClass int RTCgetCamParam2( int desc, int chno, 
			int* dipth, int* rate, int* width, int* height, int* reclen);

/** RTCgetChannelKeyParam
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] chno     Channel number
 * @param [in] key      Key name of the pParameter
 * @param [out] buffer  value of the pParameter
 * @param [in] buffer   buffer size   
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 * @note
 *      An error occurs when the experimental state is "Idle".
 *
 */
 DllClass int RTCgetChannelKeyParam( int desc, int chno, const char*key, char* buffer, int buffer_size);
/** RTCtransfer
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] action   Action code. Set to 1 to start transfer, 0 to stop transfer.
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 */
 DllClass int RTCtransfer( int desc, int action);

/** RTCgetChannelData
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] chno     Channel number
 * @param [in] timeout  Data get timeout value(msec) if <1, no timeout.
 * @param [out] status  Current status value( 0/3/5/8/9:Idle/Setup/Init/Measure/End ) 
 * @param [out] shotno  Current Shot number  
 * @param [out] subshot Current Sub-Shot number  
 * @param [out] blkno   Data block number 
 * @param [out] headno  Number of the first data in the buffer  
 * @param [out] tailno  Number of the last data in the buffer
 * @param [out] buf     Acquired data. It is necessary to secure an appropriate area in advance.  
 * @param [out] len     Length of acquired data  
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 */
 DllClass int RTCgetChannelData( int desc, int chno, int timeout,
                                 int *status, int *shotno, int *subshot,
								 int *blkno, int64_t *headno, int64_t *tailno, void* buf, int *len );

/** RTCgetYUVdata2RGB
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 * @param [in] chno     Channel number
 * @param [in] timeout  Data get timeout value(msec) if <1, no timeout.
 * @param [out] status  Current status value( 0/3/5/8/9:Idle/Setup/Init/Measure/End ) 
 * @param [out] shotno  Current Shot number  
 * @param [out] subshot Current Sub-Shot number  
 * @param [out] blkno   Data block number 
 * @param [out] headno  Number of the first data in the buffer  
 * @param [out] tailno  Number of the last data in the buffer
 * @param [out] buf     Acquired data. It is necessary to secure an appropriate area in advance.  
 * @param [out] len     Length of acquired data  
 * @param [in] fmt      YUV format 0:UYVY(basler), 1:YUY2  
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 * @date 2020-05-25
 *
 */
 DllClass int RTCgetYUVdata2RGB( int desc, int chno, int timeout,
                                 int *status, int *shotno, int *subshot,
								 int *blkno, int64_t *headno, int64_t *tailno, char* buf, int *len,
								 int fmt);
/** RTCclose
 *
 * @param [in] desc		Descriptor returned by RTCopen().
 *
 * @return If == 0, it is a success.
 *         If <0, an error has occurred.
 *
 */
 DllClass int RTCclose( int desc);

 DllClass const char* RTCserverName( int desc);
 DllClass int RTCnumChannels( int desc);
 DllClass int RTCgetChannelNumbers( int desc, int* chs, int chs_size);
 DllClass void RTCerrorVerbose( int sw);
 DllClass const char* RTCerrorMessage( int code);

 //  ----- PV-WAVE -----
 DllClass pvw_long PvwRTCStart(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCEnd(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCopen(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCchannel(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCgetChannelParam(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCgetChannelKeyParam(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCtransfer(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCgetChannelData(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCclose(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCserverName(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCnumChannels(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCgetChannelNumbers(int argc, LPVOID **argv);
 DllClass pvw_long PvwRTCerrorVerbose(int argc, LPVOID **argv);
 DllClass const char* PvwRTCerrorMessage(int argc, LPVOID **argv);

 // PvwRTCcameraStart()	-> PvwRTCStart()
 // PvwRTCcameraEnd()	-> PvwRTCEnd()
 // PvwRTCcameraOpen()	-> PvwRTCopen()
 // PvwRTCcameraClose()	-> PvwRTCclose()
 // PvwRTCcameraSetInf() n/a
 DllClass	pvw_long	PvwRTCgetCamParam(int argc, LPVOID **argv);
 //  PvwRTCcameraChannelData()	-> PvwRTCgetChannelData()
 DllClass	pvw_long	PvwRTCcamYUVdata2RGB(int argc, LPVOID **argv);

 //  ----- IDL -----
 DllClass idl_long IdlRTCStart(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCEnd(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCopen(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCchannel(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCgetChannelParam(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCgetChannelKeyParam(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCtransfer(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCgetChannelData(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCclose(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCserverName(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCnumChannels(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCgetChannelNumbers(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCerrorVerbose(int argc, LPVOID argv[]);
 DllClass const char* IdlRTCerrorMessage(int argc, LPVOID argv[]);

 DllClass idl_long IdlRTCgetCamParam(int argc, LPVOID argv[]);
 DllClass idl_long IdlRTCcamYUVdata2RGB(int argc, LPVOID argv[]);

 // Utility
 DllClass	void LRTbgrx2rgb4matlab(void *dst, void* src,int width, int height); // RTTrans2.cpp
 DllClass	void LRTyuv2rgb4matlab(void *dst, void *src, int size_x, int size_y, int fmt);	
 DllClass	void LRTmemcpy(void *dst, int64_t d_stt, int64_t d_end, void* src, int64_t len);
#ifdef __cplusplus
}
#endif

#ifndef EXCLUDE_LRT_FUNCTIONS
//------------------------------------------------------------------------------
//   AAPI (Ver. 2.0)
//------------------------------------------------------------------------------
typedef struct	lrtinf	{
	int		thinning;	//	Ԉ@1/N (1`)
} LRTINF;

typedef struct	lrtparam	{
	int		samplingInterval;	//@TvOԊuiimbj
	int		valueLen;				//@f[^rbg
	int		recLen;				//@f[^擾ubNiBytej
	int		packetSize;			//  pPbgTCY(Byte)
} LRTPARAM;

typedef struct	lrtcameraparam	{
	int		pixelDipth;			//@f[^rbg
	int		frameRate;			//@t[[giFrames/sec.j
	int		frameWidth;			//	t[
	int		frameHeight;			//	t[
	int		recLen;				//@f[^擾ubNiBytej
	int		packetSize;			//  pPbgTCY(Byte)
} LRTCAMERAPARAM;

struct	RT_TransClient_HANDLE {
	char	*DiagName;
	char	*ServerName;
	SOCKET	hsocket;
	SOCKET	dsocket;

	int		shotNo;
	int		subshotNo;
	int		channelNo;
	int		blockNo;

	int		thinningRate;
	int		samplingInterval;
	int		valueLen;
	int		recLen;
	int		packetSize;

	int		frameRate;
	int		frameWidth;
	int		frameHeight;

	int		tcpNo;
	int		udpNo;
	BOOL	inDataTransfer;
#ifdef _cplusplus
 #if defined(WIN32) || defined(WIN64)
	RT_TransClient_HANDLE() {DiagName=ServerName=NULL;hsocket=dsocket=NULL;inDataTransfer=FALSE;};
 #else
	RT_TransClient_HANDLE() {DiagName=ServerName=NULL;hsocket=dsocket=INVALID_SOCKET;inDataTransfer=FALSE;};
 #endif
#endif
} ;


/***************************************************
	DLLĂяo֐Q
****************************************************/
#ifdef __cplusplus
extern "C"
{
#endif
DllClass	pvw_long	PvwLRTStart(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTEnd(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTOpen(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTClose(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTSetInf(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTGetChannelParam(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTGetChannelData(int argc, LPVOID **argv);

DllClass	pvw_long	PvwLRTCameraStart(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraEnd(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraOpen(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraClose(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraSetInf(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraGetChannelParam(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTCameraGetChannelData(int argc, LPVOID **argv);
DllClass	pvw_long	PvwLRTYUVCameraGetChannelData(int argc, LPVOID **argv);

DllClass	pvw_long	PvwLRTGetChParam(int argc, LPVOID **argv);

DllClass	int		IdlLRTStart(int argc, LPVOID argv[]);
DllClass	int		IdlLRTEnd(int argc, LPVOID argv[]);
DllClass	int		IdlLRTOpen(int argc, LPVOID argv[]);
DllClass	int		IdlLRTClose(int argc, LPVOID argv[]);
DllClass	int		IdlLRTSetInf(int argc, LPVOID argv[]);
DllClass	int		IdlLRTGetChannelParam(int argc, LPVOID argv[]);
DllClass	int		IdlLRTGetChannelData(int argc, LPVOID argv[]);

DllClass	int		IdlLRTCameraStart(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraEnd(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraOpen(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraClose(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraSetInf(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraGetChannelParam(int argc, LPVOID argv[]);
DllClass	int		IdlLRTCameraGetChannelData(int argc, LPVOID argv[]);
DllClass	int		IdlLRTYUVCameraGetChannelData(int argc, LPVOID argv[]);

DllClass	int		IdlLRTGetChParam(int argc, LPVOID argv[]);

DllClass	int		LRTStart();
DllClass	int		LRTEnd();

#ifdef __cplusplus
 DllClass	int		LRTOpen(char *diagname, int chno, LRTINF inf, void *hnd, int udpp = 0, int packetsize=0);
#else
 DllClass	int		LRTOpen(char *diagname, int chno, LRTINF inf, void *hnd, int udpp, int packetsize);
#endif
DllClass	int		LRTClose(void *hnd);

DllClass	int		LRTSetInf(void *hnd,  LRTINF inf);
DllClass	int		LRTGetChannelParam(void *hnd, int *status, LRTPARAM *prm);
DllClass	int		LRTGetChannelParam2(void *hnd, int *status,
						int* samp_interval, int* value_len, int* rec_len,int* packet_size);

DllClass	int		LRTGetChannelData(void *hnd, int *status,
									  int *shotno, int *subshot,int *blkno, int64_t *head, int64_t *tail,
									  void *buf, int *len);
DllClass	int		LRTCameraStart();
DllClass	int		LRTCameraEnd();

#ifdef __cplusplus
 DllClass	int	LRTCameraOpen(char *diagname, int chno, LRTINF inf, void *hnd, int udpp = 0, int packetsize=0);
 DllClass	int	LRTCameraOpen2(char *diagname, int chno, LRTINF inf, void *hnd, int udpp = 0, int packetsize=0, const char* host=NULL);
#else
 DllClass	int	LRTCameraOpen(char *diagname, int chno, LRTINF inf, void *hnd, int udpp, int packetsize);
 DllClass	int	LRTCameraOpen2(char *diagname, int chno, LRTINF inf, void *hnd, int udpp, int packetsize, const char* host);
#endif
DllClass	int		LRTCameraClose(void *hnd);

DllClass	int		LRTCameraSetInf(void *hnd,  LRTINF inf);
DllClass	int		LRTCameraGetChannelParam(void *hnd, int *status, LRTCAMERAPARAM *prm);
DllClass	int		LRTCameraGetChannelParam2(void *hnd, int *status,
		int* pixelDipth, int* frameRate, int* frameWidth, int* frameHeight, int* recLen,int* packetSize);

DllClass	int		LRTCameraGetChannelData(void *hnd, int *status,
									  int *shotno, int *subshot,int *blkno, int64_t *head, int64_t *tail,
									  void *buf, int *len);
DllClass	int		LRTYUVCameraGetChannelData(void *hnd, int *status,
									  int *shotno, int *subshot,int *blkno, int64_t *head, int64_t *tail,
									  void *buf, int *len);

DllClass	int		LRTGetChParamsNum(void *hnd, int *paramsNum);
DllClass	int		LRTGetChAllParams(void *hnd, char **Key, char **Value);
DllClass	int		LRTGetChParam(void *hnd, char *Key, char *Value);
DllClass	void	LRTset_verbose( int sw );
DllClass	int		LRTget_last_error( void );

DllClass	int		LRTOpen2(char *diagname, int chno, LRTINF inf, void *hnd, int udpp, int packetsize, const char* host);

#ifdef __cplusplus
}
#endif
//------------------------------------------------------------------------------
//   ܂ŁAAPI (Ver. 2.0)
//------------------------------------------------------------------------------
#endif // EXCLUDE_LRT_FUNCTIONS

#endif	//	RTTRANSCLIENTDLL_H
