function code = changeSite( site_name , varargin)
% Change Site for retrieve.
%   chnagesite site_name
%   code = changeSite( site_name )
%   code = changeSite( site_name, 'silent' )
%
%       site_name(text)  : Site Name
%
%       code(numeric)    : result code
%           If code is 0, the result is success; otherwise is an error.
%                        -9 : cannot load shard library.
%
% OPTIONS :
%         'silent'       : do not display messages.
%
% Examples :
%   >> code = changeSite('lhd')
%   code =
%        0
%   >> changeSite lhd
%
% Last update : 2012-05-11
%
% See also
%     currentSite retrieve retrieveVersion

if nargin < 1 
   disp('Input argument "site_name" is undefined.');
   disp('Type "help changeSite".');
   return;
end
verbose = true;
nvarargin = length(varargin);
arg_check = true;
err_arg = '';

for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    end
    
    if ~ischar(varargin{idx}) 
        arg_check = false;
        break
    end
    switch varargin{idx}
        case 'silent'
            verbose = false;
        otherwise
            arg_check = false;
            err_arg = strcat(err_arg, [' ' varargin{idx}]);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal argument ' err_arg ' >']);
        disp('Type "help changeSite".');
    end
    code = -2;
    return;
end

liblabcom='libretrieve';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9;
    return;
end

%
[code,~,~,env_val] = calllib( liblabcom, 'retrieveChangeSite2', ...
    site_name ,blanks(64),blanks(32));
if code == 0
    setenv('INDEXSERVERNAME',env_val);
else
    if verbose
        retrieveMessage(code,'retrieveChangeSite2');
    end
end
end
