function code = dbsClose( dbs_des, varargin)
% Close Process for dbStore.
%   code = dbsClose( dbs_des, ...)
%
%       dbs_des(numeric)    : opened dbStore's descriptor id
%
%       code(numeric)       : status, 0 is success, other is error.
%
%  OPTIONS :
%       'verbose' : If an error occurs, output messages to display.
%
%  Example :
%       des = dbsOpen( .... );
%       if des > 0
%          ....
%          if is_error
%              dbsAbort(des);
%          else
%              dbsClose(des);
%          end
%       end
%
%  Last update : 2012-05-11
%
%  See also
%      dbsOpen dbsWrite dbsAbort dbsMessage dbstoreVersion
%      dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt

if nargout == 1
    code = -999;
end
if nargin < 1
    disp('Input argument is short.');
    disp('Type "help dbsClose".');
    return;
end

ip = inputParser;

ip.addRequired('dbs_des', @(x)isnumeric(x) && isscalar(x) && x > 0);

ip.parse(dbs_des );
% 'verbose' 
arg_check = true;
err_args = '';
verbose = false;
nvarargin = length(varargin);
for idx = 1:nvarargin
    if isempty(varargin{idx})
        continue;
    elseif ~ischar(varargin{idx}) 
        err_args = strcat(err_args,[' arg#' num2str(idx)]);
        arg_check = false;
        continue;
    end
    switch varargin{idx}
        case 'verbose'
            verbose = true;
        otherwise
            arg_check = false;
            err_args = strcat(err_args,[' "' varargin{idx} '"']);
    end
end
if( ~arg_check ) 
    if verbose
        disp(['<Error: illegal arguments :' err_args ' >']);
        disp('Type "help dbsClose".');
    end
    return;
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, verbose )
    code = -9998;
    return;
end
%
% DBSTORE_API int dbsClose(int dbs_d)
lib_func = 'dbsClose';
code = calllib( liblabcom, lib_func, dbs_des );
if code < 0
    if verbose
        dbsMessage( code, lib_func);
    end
end
end
