function ret_text = dbsMessage( code, func_name)
% Make a message from an error code of dbStore. Or display.
%   dbsMessage code
%   dbsMessage( code , func_name );
%   ret_text = dbsMessage(code, func_name)
%
%      code(numeric)    : error code
%      func_name(text)  : function name
%
%      ret_text(text)   : error message
%
% Examples :
%   >> mes = dbsMessage(-2)
%   mes =
%
%   Inappropriate number of arguments
%
%   >> dbsMessage(-3,'testFunc');
%   Error from DLL( testFunc ) ! -3 : Illegal Diagnostics name.
%   >>
%
% Last update : 2012-05-11
%
% See also
%     dbsOpen dbsWrite dbsClose dbsAbort dbstoreVersion
%     dbsOpenFrame dbsWriteFrame dbsWriteFrames dbsCloseFrame dbstore.txt

if nargout == 1
    ret_text ='';
end
if nargin == 0 
   disp('Input argument is short.');
   disp('Type "help dbsMessage".');
   return;
end
if ischar( code )
    code = str2double(code);
end
liblabcom = 'libdbstore';
if ~loadLibLabcom( liblabcom, true )
    if nargin < 2 || ~ischar(func_name)
        mes = sprintf('Error from DLL ! %d\n', code );
    else
        mes = sprintf('Error from DLL( %s ) ! %d\n', func_name, code );
    end
    if nargout < 1
        disp( mes );
    else
        ret_text = mes;
    end
    return;
end
%
% RETRIEVE_API const char* dbsErrorMessage( int code)
if nargout < 1
    mes = calllib( liblabcom, 'dbsErrorMessage', code );
    if nargin < 2 || ~ischar(func_name)
        fprintf('Error from DLL ! %d : %s\n', code, mes );
    else
        fprintf('Error from DLL( %s ) ! %d : %s\n', func_name, code, mes );
    end
else
    ret_text = calllib( liblabcom, 'dbsErrorMessage', code );
end
end