function tf_code = loadLibLabcom( lib_name, verbose )
% Load labcom shard library('libretrieve','libdbstore','librttrans').
%   loadLibLabcom lib_name
%   tf_code = loadLibLabcom( lib_name, verbose )
%       lib_name(text)  : library name
%           ('libretrieve','libdbstore','librttrans')
%       verbose(logical): If it is true, a message is displayed.
%
%       tf_code(logical): If it is true, this function is success.
%
% Examples :
%   >> loadLibLabcom 'libretrieve'
%   ans =
%         1
%
%   >> code = loadLibLabcom('libretrieve',true);
%   code =
%         1
%
% Last update : 2016-05-27
%
% See also
%     retrieve retrieve_t dbsOpen waveMonitor tvMonitor

if nargin < 1 
   disp('Input argument "lib_name" is undefined.');
   disp('Type "help loadLibLabcom".');
   if nargout > 0
       tf_code = false;
   end
   return;
end
if ~exist('verbose','var')
    verbose = true;
end
tf_code = true;
switch lib_name
    case {'libretrieve','libdbstore','librttrans'}
        if libisloaded(lib_name)
            return
        end
    otherwise
        tf_code = false;
        if verbose 
            disp(['Error load library : No support ' lib_name]);
        end
        return;
end
env_retrieve = getenv('Retrieve');
switch lib_name
    case 'libretrieve'
        if ispc
            lib_header_file = [ env_retrieve '\include\retrieve.h'];
        else
            lib_header_file = [ env_retrieve '/include/retrieve.h'];
        end
        lib_file = 'Retrieve.dll';
    case 'libdbstore'
        if ispc
            lib_header_file = [ env_retrieve '\include\dbstore.h'];
        else
            lib_header_file = [ env_retrieve '/include/dbstore.h'];
        end
        lib_file = 'dbStore.dll';
    case 'librttrans'
        if ispc
            lib_header_file = [ env_retrieve '\include\RTTransClientDLL.h'];
        else
            lib_header_file = [ env_retrieve '/include/RTTransClientDLL.h'];
        end
        lib_file = 'RTTransClientDLL.dll';
end
warning('off','all');
if exist(lib_header_file,'file') ~= 2
    tf_code = false;
    if verbose 
        disp(['Error load library : None exist ' lib_header_file]);
    end
    return;
end

archstr = computer('arch');
switch archstr
	case	'win32'
		addpath([ env_retrieve '\bin'])

	case	'win64'
		addpath([ env_retrieve '\bin64'])

	case	'glnx86'
		addpath([ env_retrieve '/bin'])

	case	'glnxa64'
		addpath([ env_retrieve '/bin64'])
        
	case	'maci64'
		addpath([ env_retrieve '/bin64'])
end
loadlibrary(lib_file, lib_header_file, 'alias', lib_name);
switch lib_name
    case 'libretrieve'
         calllib( lib_name, 'set_retrieve_t_verbose', false);
    case 'libdbstore'
    case 'librttrans'
         calllib( lib_name, 'LRTset_verbose', false);
end
end