function printParam( params )
% Display parameter for retrieve, dbStore.
%   printParam( params )
%
%      params( cell, struct ) : parameter
%
% Examples :
%   >> [data params code] = retrieve('SX8O',92000,1,1,0);
%   >> printParam(params);
%   ...
%   >> printParam(params{2});
%
%   >> [time params code] = retrieve_t('SX8O',92000,1,1,1);
%   >> printParam(params);
%
%   >> pl = initParamList();
%   >> pl = setParam(pl,'Shot',98000);
%   >> printParam(pl);
%
% Last update : 2012-5-11
%
% See also
%     getParam setParam retrieve retrieve_t retrieveVersion

if nargin ~= 1 || ~(iscell(params) || ...
        isstruct(params) && all(isfield(params ,{'Pname','Pvalue','Ptype'})) )
   disp('Input argument is short or illegal.');
   disp('Type "help printParam".');
   return;
end
fprintf('Note: Parameter value is shown first 24 characters.\n');
if iscell(params)
    c_num = length(params);
    fprintf('channel   count =  %d\n',c_num);
    if c_num < 1 
        return
    end

    for c_idx=1:c_num
        p_num = length(params{c_idx});
        fprintf('parameter count =  %d\n',p_num);
        fprintf('    %24.24s %24.24s %8s\n', 'Name', 'Value', 'Type');
        for p_idx=1:p_num
            fprintf('%3d %24.24s %24.24s %8s\n', c_idx, params{c_idx}(p_idx).Pname, ...
                params{c_idx}(p_idx).Pvalue, ...
                mParamType__Name(params{c_idx}(p_idx).Ptype));
        end
        fprintf('\n');
    end
else
    fprintf('channel   count =  %d\n',1);
    p_num = length(params);
    fprintf('parameter count =  %d\n',p_num);
    fprintf('    %24.24s %24.24s %8s\n', 'Name', 'Value', 'Type');
    for p_idx=1:p_num
        fprintf('%3d %24.24s %24.24s %8s\n', 1, params(p_idx).Pname, ...
            params(p_idx).Pvalue, mParamType__Name(params(p_idx).Ptype));
    end
    fprintf('\n');
end
end
