function ret_text = retrieveMessage( code, func_name)
% Make a message from an error code of retrieve. Or display.
%   retrieveMessage code
%   retrieveMessage( code , func_name );
%   ret_text = retrieveMessage(code, func_name)
%
%      code(numeric)    : error code
%      func_name(text)  : function name
%
%      ret_text(text)   : error message
%
% Examples :
%   >> mes = retrieveMessage(-2)
%   mes =
%
%   Inappropriate number of arguments
%
%   >> retrieveMessage(-1,'testFunc');
%   Error from DLL( testFunc ) ! -1 : Excessive number of descriptors
%   >>
%
%  Last upadte : 2012-05-11
%
% See also
%     retrieve retrieveVersion

if nargout == 1
    ret_text ='';
end
if nargin < 1 
   disp('Input argument is short.');
   disp('Type "help retrieveMessage".');
   return;
end
liblabcom = 'libretrieve';
if ~loadLibLabcom( liblabcom, true )
    return;
end
if ischar( code )
    code = str2double(code);
end
%
% RETRIEVE_API const char* retrieveErrorMessage( int code)
if nargout < 1
    mes = calllib( liblabcom, 'retrieveErrorMessage', code );
    if nargin < 2 || ~ischar(func_name)
        fprintf('Error from DLL ! %d : %s\n', code, mes );
    else
        fprintf('Error from DLL( %s ) ! %d : %s\n',func_name, code, mes );
    end
else
    ret_text = calllib( liblabcom, 'retrieveErrorMessage', code );
end
end