function [param_list,tf_code] = setParam( param_list, pname, pvalue)
% Set a parameter to parameter list for dbStore.
%   [param_list,tf_code] = setParam( param_list, pname, pvalue);
%       param_list(struct array): parameter list (see: initParamList())
%       pname(text)             : parameter name
%       pvalue(numeric,text)    : parameter value ( It is scalar.)
%
%
%       param_list(struct array): new parameter list
%       tf_code(logical)        : result code ( false is error.)
%
% Examples :
%   >> pl = initParamList();
%   >> pl = setParam(pl, 'offset', 10.0);
%   >> pl = setParam(pl, 'gain', int32(2));
%   >> pl = setParam(pl, 'status','up');
%
% Last update : 2012-05-11
%
% See also
%     initParamList printParam dbsWrite dbsCloseFrame dbstore.txt dbstoreVersion

if nargin ~= 3
    disp('Input argument is short or illegal.');
    disp('Type "help setParam".');
    return
end
tf_code = false;
if ~isstruct(param_list) || ~all(isfield(param_list,{'Pname','Ptype','Pvalue'}))
    return;
end
if ~ischar(pname)
    return;
end
if ~(ischar(pvalue) || isscalar(pvalue) )
    return;
end
n=length(param_list)+1;
ptype = mParamType__Name(class(pvalue));

switch ptype
    case 1 % char
        param_list(1,n).Pvalue = pvalue;
    case {2,3,4,7} % integer
        param_list(1,n).Pvalue = num2str(pvalue);
    case 5 % single
        param_list(1,n).Pvalue = num2str(pvalue,7);
    case 6 % double
        param_list(1,n).Pvalue = num2str(pvalue,15);
    otherwise
        return;
end
param_list(1,n).Pname = pname;
param_list(1,n).Ptype = ptype;
tf_code = true;
end
