;// 実時間データ取得プロトコルV3対応
;//   リアルタイムデータ転送  表示プロシージャ
;//                     2020-05
;//						NIFS LABCOM
;//

;//	波形用プロシージャ
PRO	Wave3Monitor,diagName,channelNumber,cport,udp=udp,nsum=nsums,thinning=thinnings,host=dashost
	usage = 0
	if( n_params(0) ne 3 ) AND ( n_params(0) ne 2) then begin
		usage = 1
	endif
	if( KEYWORD_SET(dashost) ) then begin
		ttype = size(dashost)
		if( 7 ne ttype(1) ) then begin
			usage = 1
		endif
	endif
	if( 1 eq usage ) then begin
		print, 'Usage:'

		print, '  Wave3Monitor,diagName,channelNumber[,portNumber][,/udp][,nsum=N][,thinning=N][,host=HostName]'

		print, ' '
		print, 'Input Parameters:'
		print, '  < diagName      - Diagnostics name'
		print, '  < channelNumber - Channel number'
		print, '  < portNumber    - CONTROL port number'
		print, '  < /udp          - data transfer by UDP'
		print, '  < nsum=N        - plot average of N samples'
		print, '  < thinning=T    - set transfer thinning rate to 1/T'
		print, '  < host=HostName - Host name (''DAS15'',.., etc.)'
		
		retall
	endif
	
	if( n_params(0) eq 2) then begin
		ctrl_port = long(-1)
	endif else begin
		ctrl_port = long(cport)
	endelse
	
	if (KEYWORD_SET(nsums)) then begin
		ns = nsums
	endif else begin
		ns = 1
	endelse
	if (KEYWORD_SET(thinnings)) then begin
		thin = long(thinnings)
	endif else begin
		thin = long(1)
	endelse
	RT3Retrieve,diagName,long(channelNumber),ctrl_port,udp=udp,nsum=ns,thinning=thin,host=dashost
END

;//	波形用プロシージャ
PRO	RT3Retrieve,diagName,channelNumber,ctrl_port,udp=udp,nsum=nsums,thinning=thinnings,host=dashost

	thin_type=fix(0);
	if (KEYWORD_SET(udp)) then begin
		packet_size = 0L
	endif else begin
		packet_size = -1L
	endelse
	ch_list = string(channelNumber,FORMAT='(i0)')

	print,'Terminate when "q" or "Q" is entered.'

	;//	DLLのある場所を指定
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/RTTransClientDLL.dll' $
	else object = !Retrieve + '/bin/RTTransClientDLL.dll'
	symbol_Start	= 'PvwRTCStart'
	symbol_End		= 'PvwRTCEnd'
	symbol_Open		= 'PvwRTCopen'
	symbol_GetParam = 'PvwRTCgetChannelParam'
	symbol_Close	= 'PvwRTCclose'
	symbol_GetData	= 'PvwRTCgetChannelData'
	symbol_GetChParam = 'PvwRTCgetChannelKeyParam'
	symbol_Transfer	= 'PvwRTCtransfer'
	symbol_ErrMsg	= 'PvwRTCerrorMessage'
	
	rtd = long(0)
	status = long(0)
	samplingInterval = long(0)
	valueLen = 0
	recLen = long(0)
	tmp_interval = 0L
	tmp_valuelen = 0
	tmp_reclen = 0L
	;//	サーバへの接続開始要求
	result = LINKNLOAD( object, symbol_Start)
	if (result ne 0) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, rtd, /S_Value)
		print, msg
		retall
	endif
	if( NOT KEYWORD_SET(dashost) ) then begin
		dashost=''
	endif
	
	rtd = LINKNLOAD( object, symbol_Open,$
					diagName, dashost, ch_list, ctrl_port, packet_size, thinnings, thin_type)
	if (rtd le 0) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, rtd, /S_Value)
		print, msg
		retall
	endif

	result = LINKNLOAD( object, symbol_GetParam,$
					rtd, channelNumber, samplingInterval, valueLen, recLen)
	if ( 0 GT result) then GOTO, ABORT_RT
	
	imageType = bytarr(256)
	result = LINKNLOAD( object, symbol_GetChParam,$
						rtd, channelNumber, 'ImageType', imageType, 256L)
	if ( 0 GT result) then GOTO, ABORT_RT
	
	sImageType = string(imageType)
	if( sImageType eq 'IllegalParameterName' ) then begin
		print,'Format is unkown(valueLen:' + string(valueLen,format='(I2)') + ').'
	endif else begin
		print,'Format is '+ sImageType +'(valueLen:' + string(valueLen,format='(I2)') + ').';
	endelse
	;//	表示するための配列を作成
	;//	当サンプル内ではプロットする点数を指定
	;//	下限として4000を与える
	pointnum = 4000
	;//	データ受信用バッファ作成
	;//	PvwLRTGetChannelParam を呼ぶときに、recLen にはデータパケット１つ
	;//	あたりに含まれるサンプリングデータの個数を取得しています
	if (valueLen le 8) then begin
		bytePerSample = 1
		ttype = 1; BYTE
	endif else if (valueLen le 16) then begin
		bytePerSample = 2
		ttype = 2;	INT(SHORT)
	endif else if (valueLen le 32) then begin
		bytePerSample = 4
		if( 'FLT32' eq sImageType ) then begin
			ttype = 4; FLOAT
		endif else begin
			if( !Version.release gt 9.0 ) then begin
				ttype = 13; I32
			end else begin
				ttype = 3; LONG
			endelse
		endelse
	endif else if (valueLen le 64) then begin
		bytePerSample = 8
		if( 'FLT64' eq sImageType ) then begin
			ttype = 5; DOUBLE
		endif else begin
			ttype = 3; LONG
		endelse
	endif
	
	bufsize = recLen/bytePerSample
	if (bufsize gt pointnum) then begin
		pointnum = bufsize * 2
	endif
	buf      = make_array(bufsize, type=ttype)
	plotdata = make_array(pointnum,  type=ttype)

	window, 1
	wshow, 1, 1

	prehead=long(0)
	;//	データ受信する際に、ショット番号などを受け取るための領域確保
	shot = long(0);
	subshot = long(0);

	;//	ブロック番号は1から始まり、サーバから次のブロックが
	;//	送られてくるごとにシーケンシャルに増えていきます
	blk = long(0);
	preblk = long(-1);

	;//	受信したブロック中の先頭と最終サンプリングデータが
	;//	収集データ中で何番目にあたるかを示します
	head = long(0);
	tail = long(0);
    timeout = long(100); // msec
	;//	送られてきたブロックのサイズ
	blen = long(0);
	result = LINKNLOAD( object, symbol_Transfer, rtd, 1);
	if ( 0 GT result) then GOTO, ABORT_RT

	;//	データ受信のためのループ 
	;//for loopn=1, 10000 do begin
	repeat begin

		;//	データ受信関数を呼び出す
		result = LINKNLOAD( object, symbol_GetData, rtd, channelNumber, timeout, status, shot, subshot, blk, head, tail, buf, blen)
		if( result EQ 2 ) then begin
			print,'RTTRANSD STOPPED'
			GOTO, STOP_PRO
		endif
		if( result EQ 0 ) then begin
			;//	ブロックのバイト長から、バッファ長を計算
			;//	このサンプル内では、1サンプリングあたり2バイトと決めうちしています
			buflen = blen / bytePerSample

			;//	status は -1, 3, 5, 8 が返ってくる可能性があります
			;//	-1 は意味のあるデータ無しで、他の変数にも無意味な値が入っていますので
			;//	処理なく破棄します
			;//	プラズマデータ収集開始前はほぼすべて -1 になり、
			;//	またデータ収集中でもデータ生成待ちのときには -1 が返ってきます

			;//	3, 5 はサーバがそれぞれモジュールセットアップと初期化を行ったときに返ってきます
			;//	ショット番号、サブショット番号には有効な値が入っていますので
			;//	必要な処理があれば利用できます

			;//	8 はプラズマ計測中にデータが送られてきたときに返ってきます
			;//	他の変数にもそれぞれ有効な値が入り、
			;// 指定したバッファには受信したデータが入ります

			;//	データを受信した際の処理
			;//	このサンプルでは、エラー処理を考えず、受け取ったバッファを
			;//	次々にプロット用の配列に入れていき、スクロールさせながら
			;//	表示するようになっています
			;//	実際にはパケット取りこぼした際などのため、ブロック番号、head,tail など
			;//	を用いて取りこぼしたデータは飛ばして表示するほうが正しいものとなります
			if status NE 8 then begin
				prehead = 0
				preblk = 0
			endif
			if ( status EQ 8 ) AND ( buflen GT 0 )then begin
				if preblk LT blk then begin
					; scroll
					sx = buflen
					plotdata(0 : pointnum - sx - 1) = plotdata(sx : pointnum - 1)

					; new data
					plotdata(pointnum - buflen : pointnum - 1) = buf(0 : buflen - 1)

					plot, plotdata, nsum = nsums, Title = diagName+' SHOT#'+string(shot,format='(i0)')+' CH#'+string(channelNumber,format='(i0)')+' Event#'+string(blk,format='(i0)')
					wait, 0.01

					prehead = head
					preblk = blk
				endif
			endif else begin
				print,shot, subshot,status,blk,blen
				if ( status EQ 3 ) then begin
					status = LINKNLOAD( object, symbol_GetParam,$
							rtd, channelNumber, tmp_interval, tmp_valuelen, tmp_reclen)
					if (status ge 0 AND recLen NE tmp_reclen) then begin
						recLen=tmp_reclen
						bufsize=recLen/bytePerSample
						buf    = make_array(bufsize, type=ttype)
						if (bufsize GT pointnum) then begin
							pointnum = bufsize * 2
							plotdata = make_array(pointnum,  type=ttype)
						endif
					endif else if (status LT 0) then begin
						msg = LINKNLOAD( object, symbol_ErrMsg, status, /S_Value)
						print, msg
						GOTO, STOP_PRO
					endif
				endif
			endelse
		endif
		quit = GET_KBRD(0)
	ENDREP UNTIL STRLOWCASE(quit) EQ 'q' 
;//	endfor

STOP_PRO:
	result = LINKNLOAD( object, symbol_Transfer, rtd, 0);
	;//	通信を終える際の処理
	;//	サーバとの接続クローズ
ABORT_RT:
	if ( 0 GT result) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, result, /S_Value)
		print, msg
	endif

	result = LINKNLOAD( object, symbol_Close, rtd)

	;//	DLL 内の終了処理を指示
	result = LINKNLOAD( object, symbol_End)
END

;// カメラ用プロシージャ
PRO	TV3Monitor,diagName,channelNumber,cport,udp=udp,thinning=thinnings,mode=imageformat,host=dashost,adjust=adj
	usage = 0
	if( n_params(0) ne 3 ) AND ( n_params(0) ne 2) then begin
		usage = 1
	endif
	if( KEYWORD_SET(dashost) ) then begin
		ttype = size(dashost)
		if( 7 ne ttype(1) ) then begin
			usage = 1
		endif
	endif
	if( KEYWORD_SET(imageformat) ) then begin
		ttype = size(imageformat)
		if( 7 ne ttype(1) ) then begin
			usage = 1
		endif
	endif
	if( usage eq 1) then begin
		print, 'Usage:'
		print, '  TV3Monitor,diagName,channelNumber[,portNumber][,thinning=N],[mode=ModeName][host=HostName]'
		print, ' '
		print, 'Input Parameters:'
		print, '  < diagName      - Diagnostics name'
		print, '  < channelNumber - Channel number'
		print, '  < portNumber    - CONTROL port number'
		print, '  < /udp          - data transfer by UDP'
		print, '  < thinning=N    - set transfer thinning rate to 1/N'
		print, '  < mode=ModeName - camera image format (''GRAYn''(default:GRAY8), ''RGB'', ''UYVY'', ''YUY2'')'
		print, '  < host=HostName - Host name (''DAS15'',.., etc.)'
		retall
	endif
	
	if( n_params(0) eq 2) then begin
		ctrl_port = long(-1)
	endif else begin
		ctrl_port = long(cport)
	endelse
	if (KEYWORD_SET(thinnings)) then begin
		thin = long(thinnings)
	endif else begin
		thin = long(1)
	endelse
	
	RT3CamRetrieve,diagName,long(channelNumber),ctrl_port,udp=udp,thinning=thin,mode=modename,host=dashost,adjust=adj
END
;//	カメラ用プロシージャ
PRO	RT3CamRetrieve,diagName,channelNumber,ctrl_port,udp=udp,thinning=thinnings,mode=imageformat,host=dashost,adjust=adj
	
	thin_type=fix(0);
	if (KEYWORD_SET(udp)) then begin
		packet_size = 0L
	endif else begin
		packet_size = -1L
	endelse
	ch_list = string(channelNumber,FORMAT='(i0)')

	;//	DLLのある場所を指定
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/RTTransClientDLL.dll' $
	else object = !Retrieve + '/bin/RTTransClientDLL.dll'

	;//	それぞれのDLL関数名を指定
	symbol_Start	= 'PvwRTCStart'
	symbol_End		= 'PvwRTCEnd'
	symbol_Open		= 'PvwRTCopen'
	symbol_Close	= 'PvwRTCclose'
	symbol_GetParam = 'PvwRTCgetCamParam'
	symbol_GetData	= 'PvwRTCgetChannelData'
	symbol_GetDataFromYUVCamera	= 'PvwRTCcamYUVdata2RGB'
	symbol_Transfer	= 'PvwRTCtransfer'
	symbol_ErrMsg	= 'PvwRTCerrorMessage'
	symbol_Debug	= 'PvwRTCerrorVerbose'
	symbol_GetChParam = 'PvwRTCgetChannelKeyParam'
	
;	result = LINKNLOAD( object, symbol_Debug, 1)

	;//	DLL関数にアクセスし、値を取得したい変数の領域確保
	rtd = long(0)
	status = long(0)
	pixelDepth = 0
	frameRate = 0
	frameWidth = 0
	frameHeight = 0
	recLen = long(0)
	packetSize = long(0)

	;//	サーバへの接続開始要求
	result = LINKNLOAD( object, symbol_Start)
	if (result NE 0) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, rtd, /S_Value)
		print, msg
		retall
	endif
	if( NOT KEYWORD_SET(dashost) ) then begin
		dashost=''
	endif
	
	rtd = LINKNLOAD( object, symbol_Open,$
					diagName, dashost, ch_list, ctrl_port, packet_size, thinnings, thin_type)
	if (rtd LE 0) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, rtd, /S_Value)
		print, msg
		retall
	endif
	if (NOT KEYWORD_SET(imageformat)) then begin
		imageType = bytarr(256)
		result = LINKNLOAD( object, symbol_GetChParam,$
							rtd, channelNumber, 'ImageType', imageType, 256L)
		if (0 GT result) then GOTO, ABORT_PROC
		imageformat = string(imageType)
	endif

	cameramode = 1	;//	default:GRAY8
	arraybits=8
	if(STRUPCASE(imageformat) EQ 'GRAY8') then begin
		print, 'Image Type [GRAY8]'
		arraybits=8
	endif else if(STRUPCASE(imageformat) EQ 'GRAY9') then begin
		print, 'Image Type [GRAY9]'
		arraybits=9
	endif else if(STRUPCASE(imageformat) EQ 'GRAY10') then begin
		print, 'Image Type [GRAY10]'
		arraybits=10
	endif else if(STRUPCASE(imageformat) EQ 'GRAY11') then begin
		print, 'Image Type [GRAY11]'
		arraybits=11
	endif else if(STRUPCASE(imageformat) EQ 'GRAY12') then begin
		print, 'Image Type [GRAY12]'
		arraybits=12
	endif else if(STRUPCASE(imageformat) EQ 'GRAY13') then begin
		print, 'Image Type [GRAY13]'
		arraybits=13
	endif else if(STRUPCASE(imageformat) EQ 'GRAY14') then begin
		print, 'Image Type [GRAY14]'
		arraybits=14
	endif else if(STRUPCASE(imageformat) EQ 'GRAY15') then begin
		print, 'Image Type [GRAY15]'
		arraybits=15
	endif else if(STRUPCASE(imageformat) EQ 'GRAY16') then begin
		print, 'Image Type [GRAY16]'
		arraybits=16
	endif else if(STRUPCASE(imageformat) EQ 'RGB') then begin
		print, 'Image Type [RGB]'
		cameramode = 2
		Rseq = 0
		Gseq = 1
		Bseq = 2
	endif else if(STRUPCASE(imageformat) EQ 'RBG') then begin
		print, 'Image Type [RBG]'
		cameramode = 2
		Rseq = 0
		Gseq = 2
		Bseq = 1
	endif else if(STRUPCASE(imageformat) EQ 'GRB') then begin
		print, 'Image Type [GRB]'
		cameramode = 2
		Rseq = 1
		Gseq = 0
		Bseq = 2
	endif else if(STRUPCASE(imageformat) EQ 'GBR') then begin
		print, 'Image Type [GBR]'
		cameramode = 2
		Rseq = 2
		Gseq = 0
		Bseq = 1
	endif else if(STRUPCASE(imageformat) EQ 'BRG') then begin
		print, 'Image Type [BRG]'
		cameramode = 2
		Rseq = 1
		Gseq = 2
		Bseq = 0
	endif else if(STRUPCASE(imageformat) EQ 'BGR') then begin
		print, 'Image Type [BGR]'
		cameramode = 2
		Rseq = 2
		Gseq = 1
		Bseq = 0
	endif else if(STRUPCASE(imageformat) EQ 'UYVY') then begin
		print, 'Image Type [YUV422:UYVY]'
		cameramode = 3
		yuvfmt=0
	endif else if(STRUPCASE(imageformat) EQ 'YUV422') then begin
		print, 'Image Type [YUV422:UYVY]'
		cameramode = 3
		yuvfmt=0
	endif else if(STRUPCASE(imageformat) EQ 'YUY2') then begin
		print, 'Image Type [YUV422:YUY2]'
		cameramode = 3
		yuvfmt=1
	endif else begin
		print, 'Unknown format [',imageformat,'], ignored. Considering as [GRAY8]'
		GOTO, ABORT_PROC
	endelse

	;//	通信パラメータを取得
	status = LINKNLOAD( object, symbol_GetParam,$
						rtd, channelNumber, pixelDepth, frameRate, frameWidth, frameHeight, recLen)
	if (status LT 0) then begin
		result = status
		GOTO, ABORT_PROC
	endif
	prePixelDepth = pixelDepth
	preFrameWidth = frameWidth
	preFrameHeight = frameHeight

	;//	データ受信用バッファ作成
	if(cameramode EQ 1) then begin
		if (arraybits LE 8) then begin
			buf = bytarr(frameWidth, frameHeight)
		endif else begin
			buf = intarr(frameWidth, frameHeight)
		endelse
	endif else if(cameramode EQ 2) then begin
		buf = bytarr(4, frameWidth, frameHeight)
		dsp = bytarr(3, frameWidth, frameHeight)
	endif else if(cameramode EQ 3) then begin
		buf = bytarr(3, frameWidth, frameHeight)
	endif ;//	現状、YUV422からRGBへの変換のみ対応

	shot = long(0);
	subshot = long(0);
	blk = long(0);
	head = long(0);
	tail = long(0);
	result = LINKNLOAD( object, symbol_Transfer, rtd, 1);
	if (result LT 0) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, result, /S_Value)
		print, msg
		GOTO, STOP_PRO
	endif
	timeout = 500
	;//	送られてきたブロックのサイズ
	blen = long(10);
	if(KEYWORD_SET(adj)) then begin
		wx=frameWidth
		wy=frameHeight
	endif else begin
		wx=640L
		wy=LONG(wx*frameHeight/frameWidth)
	endelse
	window, 1, XSize=wx, YSize=wy
	wshow, 1, 1
	print,'Terminate when "q" or "Q" is entered.'
	;//	データ受信のためのループ
;//	for loopn=1, 10000 do begin
	repeat begin

		;//	データ受信関数を呼び出す
		if((cameramode EQ 1) OR (cameramode EQ 2)) then begin
			result = LINKNLOAD( object, symbol_GetData, rtd, channelNumber, timeout, status, shot, subshot, blk, head, tail, buf, blen)
		endif else begin
			result = LINKNLOAD( object, symbol_GetDataFromYUVCamera, rtd, channelNumber, timeout, status, shot, subshot, blk, head, tail, buf, blen, yuvfmt)
		endelse
		if( result EQ 2 ) then begin
			print,'RTTRANSD STOPPED'
			GOTO, STOP_PRO
		endif
;		print,status,blk,head,blen
		if( result EQ 0 ) then begin

			;//	status は -1, 3, 5, 8 が返ってくる可能性があります
			;//	-1 は意味のあるデータ無しで、他の変数にも無意味な値が入っていますので
			;//	処理なく破棄します
			;//	プラズマデータ収集開始前はほぼすべて -1 になり、
			;//	またデータ収集中でもデータ生成待ちのときには -1 が返ってきます

			;//	3, 5 はサーバがそれぞれモジュールセットアップと初期化を行ったときに返ってきます
			;//	ショット番号、サブショット番号には有効な値が入っていますので
			;//	必要な処理があれば利用できます
			;//	このサンプルでは何も処理していません

			;//	8 はプラズマ計測中にデータが送られてきたときに返ってきます
			;//	他の変数にもそれぞれ有効な値が入り、
			;// 指定したバッファには受信したデータが入ります


			;//	データを受信した際の処理
			;//	このサンプルでは、エラー処理を考えず、受け取ったバッファを
			;//	次々にプロット用の配列に入れていき、スクロールさせながら
			;//	表示するようになっています
			;//	実際にはパケット取りこぼした際などのため、ブロック番号、head,tail など
			;//	を用いて取りこぼしたデータは飛ばして表示するほうが正しいものとなります
			if ( blen GT 0 )then begin
				print,'frame num = ',blk
				if(cameramode EQ 2) then begin
					dsp(0, *, *)=buf(rseq,*,*)
					dsp(1, *, *)=buf(gseq,*,*)
					dsp(2, *, *)=buf(bseq,*,*)
				endif
				erase,1
				if(cameramode EQ 1) then begin
					tvscl, buf, /order
				endif else if(cameramode EQ 2) then begin
					tvscl, dsp, true=1, /order
				endif else begin
					if ( (wx NE frameWidth) OR (wy NE frameHeight) ) then begin
						dsp=CONGRID(buf, wx, wy)
						tvscl, dsp, true=1, /order
					endif else begin
						tvscl, buf, true=1, /order
					endelse
				endelse
				wshow,1,1

				wait, 0.001
			endif else if( (status EQ 3) OR (status EQ 5) ) then begin	;//	画素数などを取得しなおして処理
				result = LINKNLOAD( object, symbol_GetParam,$
									rtd, channelNumber, pixelDepth, frameRate, frameWidth, frameHeight, recLen)
				if (0  GT result) then begin
					msg = LINKNLOAD( object, symbol_ErrMsg, result, /S_Value)
					print, msg
					GOTO, STOP_PRO
				endif
				if (prePixelDepth NE pixelDepth) then begin
					print, 'The image depth has changed. Not supported, please restart.'
					GOTO, STOP_PRO
				endif
				if ( (preFrameWidth NE frameWidth) OR (preFrameHeight NE frameHeight) ) then begin
					;//	データ受信用バッファ作成
					if(cameramode EQ 1) then begin
						if (arraybits LE 8) then begin
							buf = bytarr(frameWidth, frameHeight)
						endif else begin
							buf = intarr(frameWidth, frameHeight)
						endelse
					endif
		
					if(cameramode EQ 2) then begin
						buf = bytarr(4, frameWidth, frameHeight)
						dsp = bytarr(3, frameWidth, frameHeight)
					endif
					
					if(cameramode EQ 3) then begin
						buf = bytarr(3, frameWidth, frameHeight)
					endif ;//	現状、YUV422からRGBへの変換のみ対応

					if(KEYWORD_SET(adj)) then begin
						wx=frameWidth
						wy=frameHeight
					endif else begin
						wx=640L
						wy=LONG(wx*frameHeight/frameWidth)
					endelse
					window, 1, XSize=wx, YSize=wy
					wshow, 1, 1

					prePixelDepth = pixelDepth
					preFrameWidth = frameWidth
					preFrameHeight = frameHeight
				endif
			endif
		endif
		quit = GET_KBRD(0)
	ENDREP UNTIL STRLOWCASE(quit) EQ 'q' 
STOP_PRO:	
	result = LINKNLOAD( object, symbol_Transfer, rtd, 0);
ABORT_PROC:
	if ( 0 GT result ) then begin
		msg = LINKNLOAD( object, symbol_ErrMsg, result, /S_Value)
		print, msg
	endif
	;//	通信を終える際の処理
	;//	サーバとの接続クローズ
	result = LINKNLOAD( object, symbol_Close, rtd)

	;//	DLL 内の終了処理を指示
	result = LINKNLOAD( object, symbol_End)
END

