;// dbstore procedure for PV-WAVE ( with dbstore.dll )
;// 				Copylight 2002.2 NIFS-LABCOM
;// 				Copylight 2007.4 NIFS-LABCOM
;// Author: M.Yoshida
;// Modify: s.imazu
;//
;// Modification Histories
;// 2002/02/26 Create(M.Yoshida)
;// 2002/12/13 Update(M.Yoshida)
;// 2007/04/16 Update(s.imazu)
;// 2008/04/16 Update(s.imazu)  x64
;// 2008/09/03 Update(s.imazu)  dll update(multi-site,standard-FTP)
;// 2010/04/08 Update(Imazu S.) update DLL 13.2.1 zlib 1.2.4, zip64, DBSTORE_TMP, (INT64)
;// 2010/08/30 Update(Imazu S.) update DLL gethostbyname()
;// 2011/01/05 Update(Imazu S.) add dbsWriteFrames(PRO)
;// 2011/06/13(Ohsuna M.)   	Ver.15.0.0 x86 x86_64 インストーラ統合
;// 2012/07/31(Imazu S.)		dbsOpen,,,cndb_timeout=,cntb_retry=
;// 							dbsOpenFrame,,,/Accept_Sizes
;// 2014/10/10(Imazu S.)		x64 INT32型対応
;//

FUNCTION DBS_CAL_DATA_BYTES ,pDataArray
	size_data	= SIZE(pDataArray)
	type_data	= size_data(size_data(0)+1)
	total_data	= size_data(size_data(0)+2);
	
	CASE type_data OF
		1 : onearraybyte = 1
		2 : onearraybyte = 2
		3 : if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then onearraybyte = 8 $
   			else  onearraybyte = 4
		4 : onearraybyte = 4
		5 : onearraybyte = 8
		13 : onearraybyte = 4
	 ELSE : onearraybyte = 2 ;// undefined -> SHORT INT
	ENDCASE
	return,LONG(total_data * onearraybyte)	;// total bytes of array
END
;//
;// print dbStore Version
;//
;//		Parameter(in) : 
;//
;//				(out) :	ver_mes   ->return version message
;//
PRO dbsVersion, ver_mes
	symbol_version = 'PvwdbsVersion'

	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	result = LINKNLOAD( object, symbol_version ,/S_Value)
	if( n_params(0) eq 0 ) then begin
		print,result
	endif else begin
		ver_mes = result
	endelse
	return
END

;//
;// print message or convert to message from error code for dbstore
;//
FUNCTION dbs_message, errorCode
    if( 0  NE N_elements(errorCode) ) then begin
		if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
		else object = !Retrieve + '/bin/dbstore.dll'
		symbol_dbsErrorMessage  = 'PvwdbsErrorMessage'
		err_code = long(errorCode)
		return,LINKNLOAD( object, symbol_dbsErrorMessage, err_code, /S_Value)
	endif
	return,''
END
PRO dbs_message, errorCode, error_mes

	if ( n_params(0) ne 1 AND n_params(0) ne 2) then begin
		print, 'Usage:'
		print, ' dbs_message, errorCode [, error_mes]'
		print, ' '
		print, ' Input Arguements:'
		print, '   < errorCode   -- error code (e.g. -10)'
		print, ' Output Arguements:'
		print, '   > error_mes   -- (optinal) error message'
		print, ' '
		return
	endif

	if( n_params(0) eq 1 ) then begin
		print,dbs_message(errorCode)
	endif else begin
		error_mes = dbs_message(errorCode)
	endelse

END

;//
;// Open for dbstore
;//
FUNCTION dbsOpen, MailAddress, DiagName, ShotNumber, SubShotNumber, DataType, cndb_timeout=dbtimeout, cndb_retry=dbretry

	if ( n_params(0) ne 5 ) then begin
		return,-5;
	endif
	
	timeout_val = fix(5)
	retry_val = fix(4)
	if (KEYWORD_SET(dbtimeout)) then begin
		timeout_val = fix(dbtimeout)
	endif
	if (KEYWORD_SET(dbretry)) then begin
		retry_val = fix(dbretry)
	endif

;// defined dataType for paramType
	Type = strarr(2)
	Type(0) = 'RAW'
	Type(1) = 'ANA'

	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'
	symbol_dbsOpen  = 'PvwdbsOpen'

	Mail_Address     = MailAddress
	Diag_Name        = DiagName
	AShot_Number     = long( ShotNumber )
	ASub_Shot_Number = fix( SubShotNumber )
	Data_Type        = 0

	;// data type convert
	for cc = 0, 1 do begin
		if ( DataType EQ Type(cc) ) then begin
			Data_Type = cc + 1
		endif
	endfor

	code = LINKNLOAD( object, symbol_dbsOpen, Mail_Address, Diag_Name, AShot_Number, ASub_Shot_Number, Data_Type, timeout_val, retry_val)
	return,code
END
PRO dbsOpen, dbs_des, MailAddress, DiagName, ShotNumber, SubShotNumber, DataType, code, cndb_timeout=dbtimeout, cndb_retry=dbretry

	code = LONG(-5)						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 7 ) then begin
		print, 'Usage:'
		print, ' dbsOpen ,dbs_des ,MailAddress ,DiadgName ,ShotNumber ,SubShotNumber ,DataType ,code'
		print, ' '
		print, ' Input Arguements:'
		print, '   < MailAddress      -- Mail Address (e.g. ''xxxx@LHD.nifs.ac.jp'')'
		print, '   < DiagName         -- Diagnostics name (e.g. ''Magnetics'')'
		print, '   < ShotNumber       -- Shot Number (e.g. 32123)'
		print, '   < SubShotNumber    -- Sub Shot Number (e.g. 1)'
		print, '   < DataType         -- Data Type(''RAW'' or ''ANA'')'
		print, ' '
		print, '   < cndb_timeout=val -- Set timeout(sec) at DB connection.(default:5)'
		print, '   < cndb_retry=val   -- Set retry count after DB connection timeout.(default:3)'
		print, ' '
		print, ' Output Arguements:'
		print, '   > dbs_des          -- Descriptor(>0) or Error code(<0).'
		print, '   > code             -- =0:Success >0:Error code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		dbs_des = code
		return
	endif
	code = 0
	dbs_des = dbsOpen( MailAddress, DiagName, ShotNumber, SubShotNumber, DataType, cndb_timeout=dbtimeout, cndb_retry=dbretry )
	;// Status Check
	if ( dbs_des LT 0 ) then begin
		code = dbs_des
		print,'Error : ',dbs_message(code)
	endif
	return

END

;//
;// Open Channel Data with Frame format.
;//
FUNCTION dbsOpenFrame, dbs_des, ChannelNumber, SizeX, SizeY, ImageType, Accept_sizes=ac

	if ( n_params(0) ne 5) then begin
		return,LONG(-5);
	endif

	symbol_dbsOpenFrame = 'PvwdbsOpenFrame'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	ch_no = LONG(ChannelNumber)
	size_x = LONG(SizeX)
	size_y = LONG(SizeY)
	image_type = ImageType
	if (NOT KEYWORD_SET(ac)) then begin
		ac = LONG(0)
	endif

	code = LINKNLOAD( object, symbol_dbsOpenFrame, dbs_des, ch_no, size_x, size_y, image_type, ac )
	return,code
END
PRO dbsOpenFrame, dbs_des, ChannelNumber, SizeX, SizeY, ImageType, code, Accept_sizes=ac

	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 6) then begin
		print, 'Usage:'
		print, 'dbsOpenFrame, dbs_des, ChannelNumber, SizeX, SizeY, ImageType, code'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des        -- Opened dbs descriptor'
		print, '  < ChannelNumber  -- Channel Number'
		print, '  < SizeX          -- Width of Acquisition''s Window'
		print, '  < SizeY          -- Height of Acquisition''s Window'
		print, '  < ImageType      -- Image type of frame Data '
    if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then begin
		print, '         (''INT8'',''INT16'',''INT32'',''INT64''(only 64bit platform),''FLT32''(FLOAT),''FLT64''(DOUBLE))'
	endif else begin
		print, '         (''INT8'',''INT16'',''INT32'',''FLT32''(FLOAT),''FLT64''(DOUBLE))'
    endelse
		print, '         (''GRAY8''-''GRAY16'',''GREY8''-''GREY16'',''RGB'',''RGBA'')'
		print, ' '
		print, '  < /accept_sizes  -- Accept frame data of illegal size.'
		print, ' '
		print, 'Output Arguements:'
		print, '  > code          -- Return Code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		return
	endif

	code = dbsOpenFrame(dbs_des, ChannelNumber, SizeX, SizeY, ImageType, Accept_sizes=ac)

	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif
END

;//
;// Write Frame Data 
;//
FUNCTION dbsWriteFrame, dbs_des, ChannelNumber,FrameNumber, pDataArray
	if ( n_params(0) ne 4) then begin
		return,LONG(-5);
	endif
	symbol_dbsWriteFrame = 'PvwdbsWriteFrame'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	ch_no		= LONG(ChannelNumber)
	frame_no	= LONG(FrameNumber)
	data_bytes	= dbs_cal_data_bytes(pDataArray)
	if( data_bytes eq -1 ) then begin
		return,LONG(-1);
    endif
	code = LINKNLOAD( object, symbol_dbsWriteFrame, dbs_des,ch_no, frame_no, data_bytes, pDataArray )
	return,code
END
PRO dbsWriteFrame, dbs_des, ChannelNumber,FrameNumber, pDataArray, code

	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 5) then begin
		print, 'Usage:'
		print, 'dbsWriteFrame, dbs_des, ChannelNumber, FrameNumber, pDataArray, code'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des        -- Opened dbs descriptor'
		print, '  < ChannelNumber  -- Channel Number'
		print, '  < FrameNumber    -- Frame Number'
		print, '  < pDataArray     -- Frame Data'
		print, 'Output Arguements:'
		print, '  > code           -- Return Code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		return
	endif

	code = dbsWriteFrame (dbs_des, ChannelNumber,FrameNumber, pDataArray)
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif

END
FUNCTION dbsWriteFrames, dbs_des, ChannelNumber,StartFrame,NumFrame,pDataArray
	if ( n_params(0) ne 5) then begin
		return,LONG(-5);
	endif
	symbol_dbsWriteFrames = 'PvwdbsWriteFrames'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	ch_no		= LONG(ChannelNumber)
	start_frame	= LONG(StartFrame)
	num_frame	= LONG(NumFrame)
	data_bytes	= dbs_cal_data_bytes(pDataArray)
	if( data_bytes eq -1 ) then begin
		return,LONG(-1);
    endif

	if( 0 NE data_bytes ) then begin
		code = LINKNLOAD( object, symbol_dbsWriteFrames, dbs_des,ch_no, start_frame,num_frame,data_bytes, pDataArray )
	endif else begin
		code = 0;
	endelse
	return,code
END
PRO dbsWriteFrames, dbs_des, ChannelNumber, StartFrame, NumFrame, pDataArray, code

	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 6) then begin
		print, 'Usage:'
		print, 'dbsWriteFrames, dbs_des, ChannelNumber, StartFrame, NumFrame, pDataArray, code'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des        -- Opened dbs descriptor'
		print, '  < ChannelNumber  -- Channel Number'
		print, '  < StartFrame     -- Start Frame Number'
		print, '  < NumFrame       -- Number of Frames'
		print, '  < pDataArray     -- Frame Data'
		print, 'Output Arguements:'
		print, '  > code           -- Return Code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		return
	endif

	code = dbsWriteFrames (dbs_des, ChannelNumber, StartFrame, NumFrame, pDataArray)
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif

END
;//
;// Close Channel Data with Frame format
;//		Parameter(in) :	dbs_des       -> Opened dbs descriptor
;//						ChannelNumber -> Channel Number
;//						ParamList     -> Parameter List
;//				(out) :	code          ->return code
;//
FUNCTION dbsCloseFrame, dbs_des, ChannelNumber,ParamList
	if ( n_params(0) ne 3) then begin
		return,LONG(-5);
	endif
	symbol_dbsCloseFrame = 'PvwdbsCloseFrame'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	ch_no = LONG(ChannelNumber)

	size_para = SIZE(ParamList)
    num_para  = LONG(0)
    len_para  = LONG(0)
	param_byte_array = BYTARR(1,1)

	if( 1 EQ size_para(0) )then begin
		num_para = LONG(0);
		for cc =  (size_para(1) - 1),0,-3 do begin
			if ( strlen(ParamList(cc)) NE 0 ) then begin
				num_para = LONG(cc+1)
				goto,BREAK_NUM_PARA
			endif
		endfor
BREAK_NUM_PARA:
		
		if( num_para GT 0 ) then begin
			;// StringArray -> ByteArray & null-termination
			param_byte_array = [BYTE(ParamList(0:num_para-1)), BYTARR(1,num_para)]
			size_para = size(param_byte_array)
        	len_para  = LONG(size_para(1))
		endif
	endif

	code = LINKNLOAD( object, symbol_dbsCloseFrame, dbs_des, ch_no, num_para, len_para, param_byte_array )
	return,code
END
PRO dbsCloseFrame, dbs_des, ChannelNumber,ParamList,code

	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 4) then begin
		print, 'Usage:'
		print, 'dbsCloseFrame ,dbs_des ,ChannelNumber ,ParamList ,code'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des        -- Opened dbs descriptor'
		print, '  < ChannelNumber  -- Channel Number'
		print, '  < ParamList      -- Channel Parameter List'
		print, 'Output Arguements:'
		print, '  > code          -- Return Code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		return
	endif

	code = dbsCloseFrame(dbs_des, ChannelNumber,ParamList)
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif

	return
END

;//
;// Write Channel Data
;//
;//		Parameter(in) :	dbs_des       -> Opened dbs descriptor
;//						ChannelNumber -> Channel Number
;//						ParamList     -> Parameter List
;//						pDataArray    -> store data
;//						ImageType     -> store data image type
;//						frame         -> (optinal)
;//										If it is set, frame format store and value is total of frame.
;//
;//				(out) :	code          ->return code
;//


FUNCTION dbsWrite, dbs_des, ChannelNumber,ParamList, pDataArray,ImageType,frame=total_frame
	if ( n_params(0) ne 5 ) then begin
		return,LONG(-5);
	endif
	symbol_dbsWrite = 'PvwdbsWrite'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	ch_no = LONG(ChannelNumber)

	num_frame = 0
	if( keyword_set(total_frame) ) then num_frame = total_frame

	size_para = SIZE(ParamList)
    num_para  = LONG(0)
    len_para  = LONG(0)
	param_byte_array = BYTARR(1)

	if( 1 EQ size_para(0) )then begin
		num_para = LONG(0);
		for cc =  (size_para(1) - 1),0,-3 do begin
			if ( strlen(ParamList(cc)) NE 0 ) then begin
				num_para =LONG(cc+1)
				goto,BREAK_NUM_PARA2
			endif
		endfor
BREAK_NUM_PARA2:
		if( num_para GT 0 ) then begin
			;// StringArray -> ByteArray & null-termination
			param_byte_array = [BYTE(ParamList(0:num_para-1)), BYTARR(1,num_para)]
			size_para = size(param_byte_array)
        	len_para  = LONG(size_para(1))
		endif
	endif

	size_data	= SIZE(pDataArray)
	data_bytes	= dbs_cal_data_bytes(pDataArray)
	if( data_bytes eq -1 ) then begin
		return,LONG(-1);
    endif
	;
	if (0 EQ num_para AND 0 EQ data_bytes) then begin
		code = -204
	endif else begin
		if( 0 EQ num_frame ) then begin
			if( 0 EQ data_bytes ) then begin
				code = LINKNLOAD( object, symbol_dbsWrite, dbs_des,ch_no, num_para, len_para, param_byte_array,data_bytes, param_byte_array,ImageType )
			endif else begin
				code = LINKNLOAD( object, symbol_dbsWrite, dbs_des,ch_no, num_para, len_para, param_byte_array,data_bytes, pDataArray,ImageType )
			endelse
    	endif else begin
			size_x = LONG(0)
			size_y = LONG(0)
			for cc =  0,num_para-1, 3 do begin
				if ( ParamList(cc) EQ 'AcquisitionWindowWidth' ) then begin
					size_x = LONG(ParamList(cc+1))
				endif else if ( ParamList(cc) EQ 'AcquisitionWindowHeight' ) then begin
					size_y = LONG(ParamList(cc+1))
				endif
			endfor
        	if( 0 EQ size_x OR 0 EQ size_y ) then begin
				if( 2 EQ size_data(0) AND 1 EQ num_frame )then begin
					size_x	= size_data(1)
					size_y	= size_data(2);
				endif else if ( 3 EQ size_data(0) )then begin
					size_x	= size_data(1)
					size_y	= size_data(2);
				endif else begin
					return,LONG(-205)
				endelse
				
			endif
			code = dbsOpenFrame(dbs_des,ChannelNumber, size_x, size_y,ImageType)
			if( code NE 0 ) then begin
				return,code
			endif
			code = dbsWriteFrames(dbs_des,ChannelNumber,1,num_frame,pDataArray)
			if( code NE 0 ) then begin
				return,code
			endif
			param_list=ParamList
			dbs_DelParam,param_list,'AcquisitionWindowWidth'
			dbs_DelParam,param_list,'AcquisitionWindowHeight'
			code = dbsCloseFrame(dbs_des,ChannelNumber, param_list)
			if( code NE 0 ) then begin
				return,code
			endif
		endelse

	endelse

	return,code
END
PRO dbsWrite, dbs_des, ChannelNumber,ParamList, pDataArray,ImageType,code,frame=total_frame

	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 6) then begin
		print, 'Usage:'
		print, 'dbsWrite ,dbs_des ,ChannelNumber ,ParamList ,pDataArray ,ImageType ,code'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des        -- Opened dbs descriptor'
		print, '  < ChannelNumber  -- Channel Number'
		print, '  < ParamList      -- Channel Parameter List'
		print, '  < pDataArray     -- Channel Data'
		print, '  < ImageType      -- Image type of Channel Data '
    if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then begin
		print, '         (''INT8'',''INT16'',''INT32'',''INT64''(only 64bit platform),''FLT32''(FLOAT),''FLT64''(DOUBLE))'
	endif else begin
		print, '         (''INT8'',''INT16'',''INT32'',''FLT32''(FLOAT),''FLT64''(DOUBLE))'
	endelse
		print, '         (''GRAY8''-''GRAY16'',''GREY8''-''GREY16'',''RGB'',''RGBA'', etc)'
		print, 'Output Arguements:'
		print, '  > code          -- Return Code. See ''dbStoreErrorCode.htm'' file.'
		print, ' '
		return
	endif

	code = dbsWrite(dbs_des, ChannelNumber,ParamList, pDataArray,ImageType,frame=total_frame)
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif

	return
END
;//
;// Close
;//		Parameter(in) :	dbs_des  -> opened dbs descriptor
;//				(out) :	code     -> return code
;//
FUNCTION dbsClose ,dbs_des
	if ( n_params(0) ne 1) then begin
		return,LONG(-5);
	endif

	symbol_dbsClose = 'PvwdbsClose'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'

	code = LINKNLOAD( object, symbol_dbsClose, dbs_des )
	return,code
END
PRO dbsClose ,dbs_des ,code
	code = -5						;ILLEGAL_PARAM_COUNT
	if ( n_params(0) ne 1 AND n_params(0) ne 2 ) then begin
		print, 'Usage:'
		print, 'dbsClose ,dbs_des [,code]'
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des  -- Opened dbs descriptor'
		print, 'Output Arguements:'
		print, '  > code      -- Return Code. =0:Success'
		print, ' '
		return
	endif
	code = dbsClose ( dbs_des )
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif
	return
END

;//
;// Abort
;//
FUNCTION dbsAbort,dbs_des
	if ( n_params(0) ne 1 ) then begin
		return,LONG(-5);
	endif
	symbol_dbsAbort = 'PvwdbsAbort'
	if( !version.arch eq 'x86_64' or !version.arch eq 'x64') then object = !Retrieve + '/bin64/dbstore.dll' $
	else object = !Retrieve + '/bin/dbstore.dll'
	code = LINKNLOAD( object, symbol_dbsAbort, dbs_des )
	return,code
END

PRO dbsAbort,dbs_des

	if ( n_params(0) ne 1 ) then begin
		print, 'Usage:'
		print, 'dbsAbort ,dbs_des '
		print, ' '
		print, 'Input Parameter:'
		print, '  < dbs_des  -- Opened dbs descriptor'
		print, ' '
		return
	endif

	code = dbsAbort(dbs_des )
	return
END

;//
;// Initial Parameter List
;//		Parameter   (out) :	ParamList ->parameter list , STRARR (raw * elements)
;//		             (in) :	ParamNum  ->number of parameter
;//
FUNCTION InitParamList, NumParam
	if ( n_params(0) ne 1 ) then begin
		return,STRARR(0)
	endif
	return,STRARR( NumParam * 3 )
END
PRO InitParamList, ParamList, NumParam
	code = -5
	if ( n_params(0) ne 2 ) then begin
		print, 'Usage:'
		print, 'InitParamList ,ParamList ,NumParam'
		print, ' '
		print, 'Input Parameter:'
		print, '  < NumParam  -- Number of Parameter'
		print, 'Output Arguements:'
		print, '  > ParamList -- Initial Parameter list area.'
		print, '  > code      -- Return Code. =0:Success'
		print, ' '
		return
	endif

	ParamList = InitParamList( NumParam )
	return
END
;//
;// Set Parameter
;//		Parameter(in/out) :	ParamList ->parameter list , STRARR (raw * elements)
;//		             (in) :	KeyName   ->parameter name
;//							Value     ->parameter value
;//							ValType   ->parameter value type
;//				    (out) :	code      ->return code
;//
FUNCTION SetParam, ParamList, KeyName, Value, ValType

	if ( n_params(0) ne 4 ) then begin
		return,LONG(-5);
	endif

	sz = SIZE(ParamList)
	if( sz(0) NE 1 or sz(2) NE 7 )then begin
		return,LONG(-201)
    endif

	last_pos = 0;
	for cc =  0,(sz(1) - 1),3 do begin
		if ( strlen(ParamList(cc+2)) EQ 0 ) then begin
			last_pos = cc
			goto,BREAK_LAST_POS
		endif
	endfor
	return,LONG(-202)
BREAK_LAST_POS:

;// defined dataType for paramType
	dataType = strarr(8)
	dataType(0) = "UNDEFINED"
	dataType(1) = "STRING"
	dataType(2) = "BYTE"
	dataType(3) = "SHORT"
	dataType(4) = "INTEGER"
	dataType(5) = "FLOAT"
	dataType(6) = "DOUBLE"
	dataType(7) = "INT64"

	val_data_type = FIX(0)

	sz = SIZE(dataType)
	;// data type convert
	for cc = 0, (sz(1) - 1) do begin
		if ( ValType EQ dataType(cc) ) then begin
			val_data_type = cc
			goto,BREAK_TYPE
		endif
	endfor
BREAK_TYPE:

	if 0 NE val_data_type then begin
		;// 要素名設定
		ParamList(last_pos) = KeyName

		;// 値設定
		ParamList(last_pos+1) = STRING(Value)

		;// 型の設定
		ParamList(last_pos+2) = STRING( val_data_type )
		return,LONG(0)
	endif
	return,LONG(-203)
END
PRO SetParam, ParamList, KeyName, Value, ValType, code

	code = -5
	if ( n_params(0) ne 4 AND n_params(0) ne 5 ) then begin
		print, 'Usage:'
		print, 'SetParam ,ParamList ,KeyName ,Value ,ValType [,code]'
		print, ' '
		print, 'Input Parameter:'
		print, '  < ParamList -- Parameter List'
		print, '  < KeyName   -- Parameter Name'
		print, '  < Value     -- Parameter Value'
		print, '  < ValType   -- Value Type(''STRING'',''BYTE'',''SHORT'',''INTEGER'',''FLOAT'',''DOUBLE'',''INT64'')'
		print, 'Output Arguements:'
		print, '  > code      -- Return Code. =0:Success, =-1:Error'
		print, ' '
		return
	endif

	code = SetParam( ParamList, KeyName, Value, ValType )
	if ( code LT 0 ) then begin
		print,'Error : ',dbs_message(code)
	endif

	return
END
PRO dbs_DelParam, ParamList, KeyName

	if ( n_params(0) ne 2 ) then begin
		return
	endif

	sz = SIZE(ParamList)
	if( sz(0) NE 1 or sz(2) NE 7 )then begin
		return
    endif

	last_pos = sz(1);
	for cc =  0,(last_pos-1),3 do begin
		if ( ParamList(cc) EQ KeyName ) then begin
			ParamList(cc+2)=''
			for nn =  cc,(last_pos-4) do begin
				ParamList(nn)=ParamList(nn+3)
			endfor
			goto,BREAK_1
		endif
	endfor
BREAK_1:
	return
END
